@@ -184,7 +184,7 @@ what you give them.   Help stamp out software-hoarding!  */
 #include <sys/resource.h>
 #endif /* BSD4_2 */
 
-#ifdef __STDC_
+#ifdef __STDC__
 #ifndef HPUX
 /* not sure where this for NetBSD should really go
    and it probably applies to other systems */
@@ -198,13 +198,11 @@ extern char *sbrk ();
 extern void *sbrk ();
 #endif /* __STDC__ */
 
-extern char *start_of_data ();
+extern char *start_of_data (void);
 
 #ifdef BSD
-#ifndef DATA_SEG_BITS
 #define start_of_data() &etext
 #endif
-#endif
 
 #ifndef emacs
 #define start_of_data() &etext
@@ -287,7 +285,7 @@ static struct mhead *nextf[30];
 static char busy[30];
 
 /* Number of bytes of writable memory we can expect to be able to get */
-extern unsigned int lim_data;
+extern unsigned long lim_data;
 
 /* Level number of warnings already issued.
   0 -- no warnings issued.
@@ -341,7 +339,7 @@ morecore (nu)			/* ask system for more memory */
 {
   char *cp;
   int nblks;
-  unsigned int siz;
+  unsigned long siz;
   int oldmask;
 
 #ifdef BSD
@@ -527,7 +525,7 @@ malloc (n)		/* get a block */
 #ifdef rcheck
     botch ("block on free list clobbered");
 #else /* not rcheck */
-    abort ();
+    ABORT ();
 #endif /* not rcheck */
 
   /* Fill in the info, and if range checking, set up the magic numbers */
@@ -571,7 +569,7 @@ free (mem)
 
 #ifndef rcheck
     if (p -> mh_alloc != ISALLOC)
-      abort ();
+      ABORT ();
 
 #else /* rcheck */
     if (p -> mh_alloc != ISALLOC)
@@ -740,7 +738,7 @@ malloc_stats (size)
   return v;
 }
 int
-malloc_mem_used ()
+malloc_mem_used (void)
 {
   int i;
   int size_used;
@@ -759,7 +757,7 @@ malloc_mem_used ()
 }
 
 int 
-malloc_mem_free ()
+malloc_mem_free (void)
 {
   int i;
   int size_unused;
@@ -789,7 +787,7 @@ malloc_mem_free ()
 #ifdef USG
 
 static void
-get_lim_data ()
+get_lim_data (void)
 {
 #ifdef ULIMIT_BREAK_VALUE
   lim_data = ULIMIT_BREAK_VALUE;
@@ -804,7 +802,7 @@ get_lim_data ()
 #ifndef BSD4_2
 
 static void
-get_lim_data ()
+get_lim_data (void)
 {
   lim_data = vlimit (LIM_DATA, -1);
 }
@@ -812,7 +810,7 @@ get_lim_data ()
 #else /* BSD4_2 */
 
 static void
-get_lim_data ()
+get_lim_data (void)
 {
   struct rlimit XXrlimit;
 
